@@ -116,7 +116,7 @@ extern "C" {
 	typedef struct config_t {
 		char *user;	/* user logging in */
 		gboolean debug;
-		gboolean mkmountpoint;
+		gboolean mkmntpoint;
 		unsigned int volcount;
 		char luserconf[PATH_MAX + 1];
 		char fsckloop[PATH_MAX + 1];
@@ -194,6 +194,11 @@ extern "C" {
 /* ============================ add_to_argv () ============================= */
 void add_to_argv(char *argv[], int *const argc, char *const arg, fmt_ptrn_t *vinfo);
 
+/* ============================ decrypted_key () =========================== */ 
+int decrypted_key(char *const pt_fs_key, size_t * const pt_fs_key_len,
+              const char *const fs_key_path,
+              const char *const fs_key_cipher, const char *const authtok);
+
 #ifdef __cplusplus
 }
 #endif
