@@ -33,181 +33,31 @@
 #include <fstab.h>
 #elif defined(__linux__)
 #include <mntent.h>
+/* FIXME: for LOOP_ code below:
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <linux/loop.h>
+#include <linux/major.h>
+#include <sys/stat.h>
+*/
+#endif
+#ifdef HAVE_LIBCRYPTO
+#ifndef EVP_MAX_BLOCK_LENGTH
+#define EVP_MAX_BLOCK_LENGTH 32 /* some older openssl versions need this */
+#endif
+#else
+#define EVP_MAX_BLOCK_LENGTH 0  /* FIXME: this is ugly, but needed */
 #endif
 #include <sys/types.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <limits.h>
-#ifdef HAVE_LIBCRYPTO
-#include <openssl/evp.h>
-#ifndef EVP_MAX_BLOCK_LENGTH
-#define EVP_MAX_BLOCK_LENGTH 32	/* some older openssl versions need this */
-#endif
-#else
-#define EVP_MAX_BLOCK_LENGTH 0	/* FIXME: this is ugly, but needed */
-#endif				/* HAVE_LIBCRYPTO */
 #include <pam_mount.h>
 #include <libgen.h>
 
 extern gboolean debug;
 
-#ifdef HAVE_LIBCRYPTO
-/* ============================ hash_authtok () ============================ */
-/* INPUT: fp, file containing encrypted stream; cipher; authtok, the key 
- *        to unlock stream contained in fs
- * SIDE AFFECTS: hash, hash(authtok); iv, the initialization vector for fp;
- *               errors are logged
- * OUTPUT: if error 0 else 1
- */
-static int hash_authtok(FILE * const fp, const EVP_CIPHER * const cipher,
-			const char *const authtok,
-			unsigned char *const hash, unsigned char *const iv)
-{
-	const EVP_MD *md;
-	unsigned char salt[PKCS5_SALT_LEN];
-	char magic[sizeof "Salted__" - 1];
-
-	assert(fp != NULL);	/* FIXME: check if valid, open file */
-	assert(cipher != NULL);	/* FIXME: check if cipher is valid OpenSSL cipher */
-	assert(authtok != NULL);
-	assert(hash != NULL);	/* FIXME: check hash is big enough? */
-	assert(iv != NULL);	/* FIXME: check iv is big enough? */
-
-	if ((fread(magic, 1, sizeof "Salted__" - 1, fp) !=
-	     sizeof "Salted__" - 1)
-	    || (fread(salt, 1, PKCS5_SALT_LEN, fp) != PKCS5_SALT_LEN)) {
-		l0g("pam_mount: %s\n",
-		    "error reading salt from encrypted filesystem key");
-		return 0;
-	}
-	if (memcmp(magic, "Salted__", sizeof "Salted__" - 1) != 0) {
-		l0g("pam_mount: %s\n",
-		    "magic string Salted__ not in filesystem key file");
-		return 0;
-	}
-	md = EVP_md5();
-	if (EVP_BytesToKey
-	    (cipher, md, salt, authtok, strlen(authtok), 1,
-	     hash, iv) <= 0) {
-		l0g("pam_mount: %s\n", "failed to hash system password");
-		return 0;
-	}
-
-	return 1;
-}
-#endif				/* HAVE_LIBCRYPTO */
-
-/* ============================ decrypted_key () =========================== */
-/* INPUT: fs_key_path, the path to an encrypted file (efsk); fs_key_cipher, 
- *        the cipher used to encrypt the file; authtok, the key to unlock the 
- *        file at fs_key_path
- * SIDE AFFECTS: pt_fs_key points to the decrypted data from the file at 
- *               fs_key_path (fsk); pt_fs_key_len is the length of pt_fs_key; 
- *               errors are logged
- * OUTPUT: if error 0 else 1
- * NOTE: pt_fs_key must point to a memory block large enough to hold fsk
- *       (MAX_PAR + EVP_MAX_BLOCK_LENGTH -- length of ct_fs_key + one block)
- *       efsk = encrypted filesystem key (stored in filesystem)
- *       fsk = filesystem key (D(efsk))
- *       pt_fs_key will contain binary data; don't use strlen, strcpy, etc.
- *       pt_fs_key may contain trailing garbage; use pt_fs_key_len
- */
-static int
-decrypted_key(char *const pt_fs_key, size_t * const pt_fs_key_len,
-	      const char *const fs_key_path,
-	      const char *const fs_key_cipher, const char *const authtok)
-{
-/* FIXME: this function may need to be broken up and made more readable */
-#ifdef HAVE_LIBCRYPTO
-	int ret = 1;
-	size_t segment_len;
-	unsigned char ct_fs_key[MAX_PAR];	/* encrypted filesystem key. */
-	size_t ct_fs_key_len;
-	unsigned char hashed_authtok[EVP_MAX_KEY_LENGTH];	/* hash(system authtok) */
-	unsigned char iv[EVP_MAX_IV_LENGTH];
-	FILE *fs_key_fp;
-	const EVP_CIPHER *cipher;
-	EVP_CIPHER_CTX ctx;
-
-	assert(pt_fs_key != NULL);
-	assert(pt_fs_key_len != NULL);
-	assert(fs_key_cipher != NULL);	/* fs_key_cipher = D, where D_key(efsk) = fsk */
-	assert(fs_key_path != NULL);	/* path to efsk */
-	assert(authtok != NULL);	/* should unlock efsk */
-
-	memset(pt_fs_key, 0x00, MAX_PAR + EVP_MAX_BLOCK_LENGTH);
-	OpenSSL_add_all_ciphers();
-	EVP_CIPHER_CTX_init(&ctx);
-	if (!(fs_key_fp = fopen(fs_key_path, "r"))) {
-		l0g("pam_mount: error opening %s\n", fs_key_path);
-		ret = 0;
-		goto _return_no_close;
-	}
-	if (!(cipher = EVP_get_cipherbyname(fs_key_cipher))) {
-		l0g("pam_mount: error getting cipher \"%s\"\n",
-		    fs_key_cipher);
-		ret = 0;
-		goto _return;
-	}
-	if (hash_authtok(fs_key_fp, cipher, authtok, hashed_authtok, iv) ==
-	    0) {
-		ret = 0;
-		goto _return;
-	}
-	if ((ct_fs_key_len = fread(ct_fs_key, 1, MAX_PAR, fs_key_fp)) == 0) {
-		l0g("pam_mount: failed to read encrypted filesystem key from %s\n", fs_key_path);
-		ret = 0;
-		goto _return;
-	}
-	if (EVP_DecryptInit_ex(&ctx, cipher, NULL, hashed_authtok, iv) ==
-	    0) {
-		l0g("pam_mount: %s\n",
-		    "failed to initialize decryption code");
-		ret = 0;
-		goto _return;
-	}
-	/* assumes plaintexts is always <= ciphertext + EVP_MAX_BLOCK_LEN in length 
-	 * OpenSSL's documentation seems to promise this */
-	if (EVP_DecryptUpdate
-	    (&ctx, pt_fs_key, &segment_len, ct_fs_key,
-	     ct_fs_key_len) == 0) {
-		l0g("pam_mount: %s\n", "failed to decrypt key");
-		ret = 0;
-		goto _return;
-	}
-	*pt_fs_key_len = segment_len;
-	if (EVP_DecryptFinal_ex
-	    (&ctx, &pt_fs_key[*pt_fs_key_len], &segment_len) == 0) {
-		l0g("pam_mount: %s\n",
-		    "bad pad on end of encrypted file (wrong algorithm or key size?)");
-		ret = 0;
-		goto _return;
-	}
-	*pt_fs_key_len += segment_len;
-      _return:
-	if (fclose(fs_key_fp) != 0) {
-		l0g("pam_mount: error closing file pointer\n");
-		ret = 0;
-	}
-      _return_no_close:
-	if (EVP_CIPHER_CTX_cleanup(&ctx) == 0) {
-		l0g("pam_mount: error cleaning up cipher context\n");
-		ret = 0;
-	}
-
-	/* pt_fs_key_len is unsigned */
-	assert(ret == 0
-	       || *pt_fs_key_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
-
-	return ret;
-#else
-	l0g("pam_mount: %s\n",
-	    "encrypted filesystem key not supported: no openssl");
-	return 0;
-#endif				/* HAVE_LIBCRYPTO */
-}
-
 /* ============================ log_output () ============================== */
 /* INPUT: fd, a valid file descriptor
  * SIDE AFFECTS: contents of fd are logged, usually fd is connected by a 
@@ -245,6 +95,7 @@ static void run_lsof(const struct config_t *const config,
 	for (i = 0; config->command[i][LSOF]; i++)
 		add_to_argv(_argv, &_argc, config->command[i][LSOF],
 			    vinfo);
+	log_argv(_argv);
 	if (g_spawn_async_with_pipes
 	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
 	     &pid, NULL, &cstdout, NULL, &err) == FALSE) {
@@ -260,52 +111,6 @@ static void run_lsof(const struct config_t *const config,
 	CLOSE(cstdout);
 }
 
-/* ============================ mkmountpoint () ============================ */
-/* POST:   the directory named d exists && volume->created_mntpt = TRUE
- * FN VAL: if error 0 else 1, errors are logged
- */
-static int mkmountpoint(vol_t * const volume, const char *const d)
-{
-	int ret = 1;
-	struct passwd *passwd_ent;
-	char dcopy[PATH_MAX + 1], *parent;
-
-	assert(vol_t_valid(volume));
-	assert(d != NULL);
-
-	w4rn("pam_mount: creating mount point %s\n", d);
-	strncpy(dcopy, d, PATH_MAX);
-	dcopy[PATH_MAX] = (char) 0x00;
-	parent = g_dirname(dcopy);
-	if (exists(parent) == 0 && mkmountpoint(volume, parent) == 0) {
-		ret = 0;
-		goto _return;
-	}
-	if ((passwd_ent = getpwnam(volume->user))) {
-		if (mkdir(d, 0700) != 0) {
-			l0g("pam_mount: tried to create %s but failed\n",
-			    d);
-			ret = 0;
-			goto _return;
-		}
-		if (chown(d, passwd_ent->pw_uid, passwd_ent->pw_gid)
-		    != 0) {
-			l0g("pam_mount: could not chown %s to %s\n",
-			    d, volume->user);
-			ret = 0;
-			goto _return;
-		}
-	} else {
-		l0g("pam_mount: could not determine uid from %s to make %s\n", volume->user, d);
-		ret = 0;
-		goto _return;
-	}
-	volume->created_mntpt = TRUE;
-      _return:
-	g_free(parent);
-	return ret;
-}
-
 /* ============================ already_mounted () ========================= */
 /*
  * PRE:    config->volume[vol].type is a mount type (LCLMOUNT, SMBMOUNT, ...)
@@ -382,9 +187,30 @@ static int already_mounted(const struct config_t *const config,
 	}
 	w4rn("pam_mount: checking to see if %s is already mounted at %s\n",
 	     match, config->volume[vol].mountpoint);
-	while (mtab_record = getmntent(mtab))
+	while ((mtab_record = getmntent(mtab)) != NULL) {
+               char const *mnt_fsname = mtab_record->mnt_fsname;
+               struct stat statbuf;
+		/* FIXME: need to figure out where LOOP_GET_STATUS64 is from
+               if (stat(mnt_fsname, &statbuf) == 0 &&
+                    S_ISBLK(statbuf.st_mode) &&
+                    major(statbuf.st_rdev) == LOOP_MAJOR) {
+		*/
+			/* if /etc/mtab is a link to /proc/mounts then the loop
+			 * device instead of the real device will be listed --
+			 * resolve it.
+			 */
+		/*
+                       int fd = open(mnt_fsname, O_RDONLY);
+                       if (fd != -1) {
+                               struct loop_info64 loopinfo64;
+                               if (ioctl(fd, LOOP_GET_STATUS64, &loopinfo64) == 0)
+                                       mnt_fsname = loopinfo64.lo_file_name;
+                               close(fd);
+                       }
+               }
+		*/
 		/* FIXME: okay to always ignore case (needed for NCP)? */
-		if (!strcasecmp(mtab_record->mnt_fsname, match)) {
+		if (!strcasecmp(mnt_fsname, match)) {
 			strncpy(mntpt, mtab_record->mnt_dir, PATH_MAX);
 			mntpt[PATH_MAX] = 0x00;
 			mounted = 1;
@@ -396,6 +222,7 @@ static int already_mounted(const struct config_t *const config,
 				break;
 			}
 		}
+	}
 	fclose(mtab);
 	return mounted;
 #elif defined(__FreeBSD__) || defined(__OpenBSD__)
@@ -416,6 +243,7 @@ static int already_mounted(const struct config_t *const config,
 		for (i = 0; config->command[i][MNTCHECK]; i++)
 			add_to_argv(_argv, &_argc,
 				    config->command[i][MNTCHECK], vinfo);
+		log_argv(_argv);
 		if (g_spawn_async_with_pipes
 		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
 		     NULL, &pid, NULL, &cstdout, NULL, &err) == FALSE) {
@@ -500,6 +328,52 @@ static void log_pm_input(const struct config_t *const config,
 	w4rn("pam_mount: %s\n", "--------");
 }
 
+/* ============================ mkmountpoint () ============================ */
+/* POST:   the directory named d exists && volume->created_mntpt = TRUE
+ * FN VAL: if error 0 else 1, errors are logged
+ */
+static int mkmountpoint(vol_t * const volume, const char *const d)
+{
+	int ret = 1;
+	struct passwd *passwd_ent;
+	char dcopy[PATH_MAX + 1], *parent;
+
+	assert(vol_t_valid(volume));
+	assert(d != NULL);
+
+	w4rn("pam_mount: creating mount point %s\n", d);
+	strncpy(dcopy, d, PATH_MAX);
+	dcopy[PATH_MAX] = (char) 0x00;
+	parent = g_dirname(dcopy);
+	if (exists(parent) == 0 && mkmountpoint(volume, parent) == 0) {
+		ret = 0;
+		goto _return;
+	}
+	if ((passwd_ent = getpwnam(volume->user))) {
+		if (mkdir(d, 0700) != 0) {
+			l0g("pam_mount: tried to create %s but failed\n",
+			    d);
+			ret = 0;
+			goto _return;
+		}
+		if (chown(d, passwd_ent->pw_uid, passwd_ent->pw_gid)
+		    != 0) {
+			l0g("pam_mount: could not chown %s to %s\n",
+			    d, volume->user);
+			ret = 0;
+			goto _return;
+		}
+	} else {
+		l0g("pam_mount: could not determine uid from %s to make %s\n", volume->user, d);
+		ret = 0;
+		goto _return;
+	}
+	volume->created_mntpt = TRUE;
+      _return:
+	g_free(parent);
+	return ret;
+}
+
 /* ============================ do_unmount () ============================== */
 int
 do_unmount(struct config_t *config, const unsigned int vol,
@@ -508,7 +382,7 @@ do_unmount(struct config_t *config, const unsigned int vol,
 /* PRE:    config points to a valid struct config_t*
  *         config->volume[vol] is a valid struct vol_t
  *         vinfo is a valid struct fmt_ptrn_t
- *         mkmountpoint is true if mount point should be rmdir'ed
+ *         mkmntpoint is true if mount point should be rmdir'ed
  * POST:   volume is unmounted
  * FN VAL: if error 0 else 1, errors are logged
  */
@@ -539,6 +413,7 @@ do_unmount(struct config_t *config, const unsigned int vol,
 		add_to_argv(_argv, &_argc, "/usr/bin/umount.crypt", vinfo);
 		add_to_argv(_argv, &_argc, "%(MNTPT)", vinfo);
 	}
+	log_argv(_argv);
 	if (g_spawn_async_with_pipes
 	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, setrootid, NULL,
 	     &pid, NULL, NULL, &cstderr, &err) == FALSE) {
@@ -615,7 +490,7 @@ do_losetup(struct config_t *config, const unsigned int vol,
 {
 	pid_t pid;
 	GError *err = NULL;
-	int i, child_exit, _argc = 0, cstdin = -1, cstderr = -1;
+	int i, ret = 1, child_exit, _argc = 0, cstdin = -1, cstderr = -1;
 	char *_argv[MAX_PAR + 1];
 	const char *cipher =
 	    optlist_value(config->volume[vol].options, "encryption");
@@ -643,35 +518,29 @@ do_losetup(struct config_t *config, const unsigned int vol,
 		add_to_argv(_argv, &_argc,
 			    config->command[i][LOSETUP], vinfo);
 	}
-	if (cipher) {
-		if (g_spawn_async_with_pipes
-		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
-		     NULL, &pid, &cstdin, NULL, &cstderr, &err) == FALSE) {
-			l0g("pam_mount: %s\n", err->message);
-			g_error_free(err);
-			return 0;
-		}
-		if (pipewrite(cstdin, password, password_len) != password_len)
-			/* FIXME: clean: returns value of exit below */
-			l0g("pam_mount: error sending password to losetup\n");
-		CLOSE(cstdin);
-	} else {
-		if (g_spawn_async_with_pipes
-		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
-		     NULL, &pid, NULL, NULL, &cstderr, &err) == FALSE) {
-			l0g("pam_mount: %s\n", err->message);
-			g_error_free(err);
-			return 0;
-		}
+	log_argv(_argv);
+	if (g_spawn_async_with_pipes
+	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, setrootid, NULL,
+	     &pid, NULL, NULL, &cstderr, &err) == FALSE) {
+		l0g("pam_mount: %s\n", err->message);
+		g_error_free(err);
+		ret = 0;
+		goto _return;
 	}
+	w4rn("pam_mount: umount errors (should be empty):\n");
 	log_output(cstderr);
-	w4rn("pam_mount: %s\n", "waiting for losetup");
+	CLOSE(cstderr);
+	w4rn("pam_mount: %s\n", "waiting for umount");
 	if (waitpid(pid, &child_exit, 0) == -1) {
 		l0g("pam_mount: error waiting for child\n");
-		return 0;
+		ret = 0;
+		goto _return;
+	} else {
+		/* pass on through the result from the umount process */
+		ret = !WEXITSTATUS(child_exit);
 	}
-	/* pass on through the result */
-	return (!WEXITSTATUS(child_exit));
+      _return:
+	return ret;
 }
 
 /* ============================ do_unlosetup () ============================ */
@@ -699,6 +568,7 @@ int do_unlosetup(struct config_t *config, fmt_ptrn_t * vinfo)
 	for (i = 0; config->command[i][UNLOSETUP]; i++)
 		add_to_argv(_argv, &_argc,
 			    config->command[i][UNLOSETUP], vinfo);
+	log_argv(_argv);
 	if (g_spawn_async_with_pipes
 	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
 	     &pid, NULL, NULL, NULL, &err) == FALSE) {
@@ -756,6 +626,7 @@ check_filesystem(struct config_t *config, const unsigned int vol,
 	for (i = 0; config->command[i][FSCK]; i++)
 		add_to_argv(_argv, &_argc, config->command[i][FSCK],
 			    vinfo);
+	log_argv(_argv);
 	if (g_spawn_async_with_pipes
 	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
 	     &pid, NULL, &cstdout, &cstderr, &err) == FALSE) {
@@ -797,7 +668,6 @@ do_mount(struct config_t *config, const unsigned int vol,
  */
 {
 	char *_argv[MAX_PAR + 1];
-	char options[MAX_PAR + 1];
 	char prev_mntpt[PATH_MAX + 1];
 	size_t _password_len;
 	int i, mount_again = 0;
@@ -810,7 +680,7 @@ do_mount(struct config_t *config, const unsigned int vol,
 	assert(password);
 
 	/* FIXME: This is a little ugly, especially check for != LCLMOUNT */
-	if ((mount_again = already_mounted(config, vol, prev_mntpt)))
+	if ((mount_again = already_mounted(config, vol, prev_mntpt))) {
 		if (mount_again == -1) {
 			l0g("pam_mount: could not determine if %s is already mounted, failing\n", config->volume[vol].volume);
 			return 0;
@@ -828,7 +698,8 @@ do_mount(struct config_t *config, const unsigned int vol,
 			    && config->volume[vol].type != CRYPTMOUNT)
 				mount_again = 0;
 		}
-	if (!exists(config->volume[vol].mountpoint))
+	}
+	if (!exists(config->volume[vol].mountpoint)) {
 		if (mkmntpoint) {
 			if (!mkmountpoint
 			    (&config->volume[vol],
@@ -838,6 +709,7 @@ do_mount(struct config_t *config, const unsigned int vol,
 			l0g("pam_mount: mount point %s does not exist (pam_mount not configured to make it)\n", config->volume[vol].mountpoint);
 			return 0;
 		}
+	}
 	if (mount_again) {
 		GError *err = NULL;
 		if (!config->command[0][MNTAGAIN]) {
@@ -850,6 +722,7 @@ do_mount(struct config_t *config, const unsigned int vol,
 		for (i = 0; config->command[i][MNTAGAIN]; i++)
 			add_to_argv(_argv, &_argc,
 				    config->command[i][MNTAGAIN], vinfo);
+		log_argv(_argv);
 		if (g_spawn_async_with_pipes
 		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
 		     setrootid, NULL, &pid, NULL, NULL, &cstderr,
@@ -906,7 +779,8 @@ do_mount(struct config_t *config, const unsigned int vol,
 			add_to_argv(_argv, &_argc,
 				    config->command[i][config->volume[vol].
 						       type], vinfo);
-		 if (config->volume[vol].type == LCLMOUNT
+		log_argv(_argv);
+		if (config->volume[vol].type == LCLMOUNT
 		    && !check_filesystem(config, vol, vinfo, _password,
 					 _password_len))
 			l0g("pam_mount: %s\n",
